#ifndef COMPILER_HEADER
#define COMPILER_HEADER

#include "TypeDefs.h"
#include "Absyn.h"
#include "Skeleton.h"
#include <filesystem>
#include <utility>
#include "codeGen/Variable.h"
#include "ParseError.h"
#include "Info.h"

class Compiler : public Skeleton
{
public:
	Compiler(const std::string& f, shared_ptr<Scope> s) : file(f), buf(), scope(std::move(s)) {};
	std::string compile(Visitable *v);

	static const std::string extension;
	static void externalCommand(std::filesystem::path file);

private:
	std::filesystem::path file;
	std::stringstream buf;
	shared_ptr<Scope> scope;

	VariablePtr lastVar;

	VariablePtr evalExpr(Visitable *expr) {
		if (!expr) throw runtime_error("No expr to eval");
		lastVar = nullptr;
		try {
			expr->accept(this);
		} catch (const ParseError &err) {
			throw ParseError(err, expr);
		}
		if (!lastVar) throw ParseError("No variable found", expr);
		auto ret = lastVar;
		lastVar = nullptr;
		return ret;
	};
};

#endif
